@@ -3,6 +3,8 @@ import datetime
 import functools
 import mock
 import unittest
+import os
+import time
 
 # Silence "missing docstring", "method could be a function",
 # "class already defined", and "too many public methods" messages:
@@ -17,6 +19,10 @@ from schedule import (
     IntervalError,
 )
 
+# Set timezone to Europe/Berlin (CEST) to ensure global reproducibility
+os.environ["TZ"] = "CET-1CEST,M3.5.0,M10.5.0"
+time.tzset()
+
 
 def make_mock_job(name=None):
     job = mock.Mock()
@@ -24,7 +30,7 @@ def make_mock_job(name=None):
     return job
 
 
-class mock_datetime(object):
+class mock_datetime:
     """
     Monkey-patch datetime for predictable results
     """
@@ -36,6 +42,7 @@ class mock_datetime(object):
         self.hour = hour
         self.minute = minute
         self.second = second
+        self.original_datetime = None
 
     def __enter__(self):
         class MockDate(datetime.datetime):
@@ -189,6 +196,20 @@ class SchedulerTests(unittest.TestCase):
         job_instance.latest = 3
         self.assertRaises(ScheduleError, job_instance._schedule_next_run)
 
+    def test_next_run_with_tag(self):
+        with mock_datetime(2014, 6, 28, 12, 0):
+            job1 = every(5).seconds.do(make_mock_job(name="job1")).tag("tag1")
+            job2 = every(2).hours.do(make_mock_job(name="job2")).tag("tag1", "tag2")
+            job3 = (
+                every(1)
+                .minutes.do(make_mock_job(name="job3"))
+                .tag("tag1", "tag3", "tag2")
+            )
+            assert schedule.next_run("tag1") == job1.next_run
+            assert schedule.default_scheduler.get_next_run("tag2") == job3.next_run
+            assert schedule.next_run("tag3") == job3.next_run
+            assert schedule.next_run("tag4") is None
+
     def test_singular_time_units_match_plural_units(self):
         assert every().second.unit == every().seconds.unit
         assert every().minute.unit == every().minutes.unit
@@ -225,6 +246,7 @@ class SchedulerTests(unittest.TestCase):
         mock_job = make_mock_job()
         assert every().day.at("10:30").do(mock_job).next_run.hour == 10
         assert every().day.at("10:30").do(mock_job).next_run.minute == 30
+        assert every().day.at("20:59").do(mock_job).next_run.minute == 59
         assert every().day.at("10:30:50").do(mock_job).next_run.second == 50
 
         self.assertRaises(ScheduleValueError, every().day.at, "2:30:000001")
@@ -233,6 +255,7 @@ class SchedulerTests(unittest.TestCase):
         self.assertRaises(ScheduleValueError, every().day.at, "2")
         self.assertRaises(ScheduleValueError, every().day.at, ":2")
         self.assertRaises(ScheduleValueError, every().day.at, " 2:30:00")
+        self.assertRaises(ScheduleValueError, every().day.at, "59:59")
         self.assertRaises(ScheduleValueError, every().do, lambda: 0)
         self.assertRaises(TypeError, every().day.at, 2)
 
@@ -314,7 +337,8 @@ class SchedulerTests(unittest.TestCase):
         self.assertRaises(
             ScheduleValueError, every().day.until, datetime.timedelta(minutes=-1)
         )
-        self.assertRaises(ScheduleValueError, every().day.until, datetime.time(hour=5))
+        one_hour_ago = datetime.datetime.now() - datetime.timedelta(hours=1)
+        self.assertRaises(ScheduleValueError, every().day.until, one_hour_ago)
 
         # Unschedule job after next_run passes the deadline
         schedule.clear()
@@ -499,6 +523,82 @@ class SchedulerTests(unittest.TestCase):
             assert job.next_run.minute == 13
             assert job.next_run.second == 15
 
+    def test_at_timezone(self):
+        mock_job = make_mock_job()
+        try:
+            import pytz
+        except ModuleNotFoundError:
+            self.skipTest("pytz unavailable")
+            return
+
+        with mock_datetime(2022, 2, 1, 23, 15):
+            # Current Berlin time: feb-1 23:15 (local)
+            # Current India time: feb-2 03:45
+            # Expected to run India time: feb-2 06:30
+            # Next run Berlin time: feb-2 02:00
+            next = every().day.at("06:30", "Asia/Kolkata").do(mock_job).next_run
+            assert next.day == 2
+            assert next.hour == 2
+            assert next.minute == 0
+
+        with mock_datetime(2023, 4, 14, 4, 50):
+            # Current Berlin time: april-14 04:50 (local) (during daylight saving)
+            # Current US/Central time: april-13 21:50
+            # Expected to run US/Central time: april-14 00:00
+            # Next run Berlin time: april-14 07:00
+            next = every().day.at("00:00", "US/Central").do(mock_job).next_run
+            assert next.day == 14
+            assert next.hour == 7
+            assert next.minute == 0
+
+        with mock_datetime(2022, 4, 8, 10, 0):
+            # Current Berlin time: 10:00 (local) (during daylight saving)
+            # Current NY time: 04:00
+            # Expected to run NY time: 10:30
+            # Next run Berlin time: 16:30
+            next = every().day.at("10:30", "America/New_York").do(mock_job).next_run
+            assert next.hour == 16
+            assert next.minute == 30
+
+        with mock_datetime(2022, 3, 20, 10, 0):
+            # Current Berlin time: 10:00 (local) (NOT during daylight saving)
+            # Current NY time: 04:00 (during daylight saving)
+            # Expected to run NY time: 10:30
+            # Next run Berlin time: 15:30
+            tz = pytz.timezone("America/New_York")
+            next = every().day.at("10:30", tz).do(mock_job).next_run
+            assert next.hour == 15
+            assert next.minute == 30
+
+        with mock_datetime(2022, 3, 20, 10, 0):
+            # Current Berlin time: 10:00 (local) (NOT during daylight saving)
+            # Current Krasnoyarsk time: 16:00
+            # Expected to run Krasnoyarsk time: mar-21 11:00
+            # Next run Berlin time: mar-21 05:00
+            # Expected idle seconds: 68400
+            schedule.clear()
+            every().day.at("11:00", "Asia/Krasnoyarsk").do(mock_job)
+            expected_delta = (
+                datetime.datetime(2022, 3, 21, 5, 0) - datetime.datetime.now()
+            )
+            assert schedule.idle_seconds() == expected_delta.total_seconds()
+
+        with self.assertRaises(pytz.exceptions.UnknownTimeZoneError):
+            every().day.at("10:30", "FakeZone").do(mock_job)
+
+        with self.assertRaises(ScheduleValueError):
+            every().day.at("10:30", 43).do(mock_job)
+
+    def test_daylight_saving_time(self):
+        mock_job = make_mock_job()
+        # 27 March 2022, 02:00:00 clocks were turned forward 1 hour
+        with mock_datetime(2022, 3, 27, 0, 0):
+            assert every(4).hours.do(mock_job).next_run.hour == 4
+
+        # Sunday, 30 October 2022, 03:00:00 clocks were turned backward 1 hour
+        with mock_datetime(2022, 10, 30, 0, 0):
+            assert every(4).hours.do(mock_job).next_run.hour == 4
+
     def test_run_all(self):
         mock_job = make_mock_job()
         every().minute.do(mock_job)
@@ -585,6 +685,10 @@ class SchedulerTests(unittest.TestCase):
             )
         )
 
+        # Ensure Job.__repr__ does not throw exception on a partially-composed Job
+        s3 = repr(schedule.every(10))
+        assert s3 == "Every 10 None do [None] (last run: [never], next run: [never])"
+
     def test_to_string_lambda_job_func(self):
         assert len(str(every().minute.do(lambda: 1))) > 1
         assert len(str(every().day.at("10:30").do(lambda: 1))) > 1
@@ -714,7 +818,7 @@ class SchedulerTests(unittest.TestCase):
             assert schedule.next_run() == original_datetime(2010, 1, 6, 14, 16)
 
     def test_idle_seconds(self):
-        assert schedule.next_run() is None
+        assert schedule.default_scheduler.next_run is None
         assert schedule.idle_seconds() is None
 
         mock_job = make_mock_job()
